import gym
import numpy as np
from .dataframe import DataFrame
from .gis import GIS
import re
from .csm import CSM


class R3(gym.Env): 
    
    # 
    MAX_SOWING_DAY = 100
    
    # action space definition
    ADD_DAY = 0
    SUB_DAY = 1
    SAME_DAY = 2
    action_space = gym.spaces.Discrete(2)
    actions = [ADD_DAY, SUB_DAY]
    
    # observation space definition
    observation_space = gym.spaces.Box(low = 0, 
                                            high = 100,
                                            shape = (33,),
                                            dtype = np.int)
    AGENT = -1
    
    def __init__(self, stochastic=True, fitness_threshold=-1000):
        r3_plots_path = "plots.shp"
        r3_pipelines_path = "pipelines.shp"
        self.layers = GIS()
        self.layers.add_data_layer(r3_plots_path, 'plots')
        self.layers.add_data_layer(r3_pipelines_path, 'pipelines')
        self.csm = CSM()
        self.sow()
        self.fitness_threshold = fitness_threshold
        
    def sow(self, sowing_dates_series=None):
        if sowing_dates_series is None:
            self.layers.add_random_series_column('pipelines', 'sowing_dates')
        else:
            self.layers.add_column('pipelines', sowing_dates_series, 'sowing_dates')
    
    def get_state(self):
        """start_state = np.where(self.grid_state == self.AGENT)
        start_not_found = not (start_state[0] and goal_state[0])
        if start_not_found:
            print("Start state not present in the Gridworld. Check the Grid layout")

        #start_state = (start_state[0][0])
        start_state = 0"""
        
        return self.layers.get_column('sowing_dates')
    
    
    def step(self, action):
        """
        Run one step into the env
        Args:
            state (Any): Current index state of the maze
            action (int): Discrete action for up, down,
            left, right
            slip (bool, optional): Stochasticity in the 
            env. Defaults to True.
        Raises:
            ValueError: If invalid action is provided as 
            input
        Returns:
            Tuple : Next state, reward, done, _
       
            return next observation, reward, done, info
        """
        
        action = int(action)
        info = {"success": True}
        self.grid_state, reward = self.get_state_reward(action)
        done = (reward > self.fitness_threshold)
        if done is False:
            reward = 1.0
        else:
            reward = -2.0
        return self.grid_state, reward, done, info
    
    def get_state_reward(self, action):
        actual_fitness = self.fitness_sowing_dates_distribution()
        random_plot = np.random.randint(0, 33)
        actual_sowing_dates = self.layers.get_column('pipelines', 'sowing_dates').to_numpy()
        #print(actual_sowing_dates)
        if action == self.ADD_DAY:
            if actual_sowing_dates[random_plot] <= 100: 
                actual_sowing_dates[random_plot] += 1
        elif action == self.SUB_DAY:
            if actual_sowing_dates[random_plot] != 0:
                actual_sowing_dates[random_plot] += -1
        elif action == self.SAME_DAY:
            pass
        #print("Taken action:", action)
        self.sow(actual_sowing_dates)
        
        #self.layers.set_row('pipelines', 'sowing_dates', random_plot, actual_sowing_dates)
        self.estimated_cluster_yield()
        reward = self.fitness_sowing_dates_distribution() - actual_fitness
        return self.layers.get_column('pipelines', 'sowing_dates').to_numpy(), reward
    
    def render(self):
        return self.show()
    
    def reset(self):
        self.sow()
        return self.layers.get_column('pipelines', 'sowing_dates').to_numpy()
        #return self.set_sowing_dates(np.ones((116, 1)))
    
    def get_sowing_dates(self):
        return self.layers.get_column('sowing_dates')
    
    def get_delta_list(self, delta=7):
        network_constraints_matrix = []
        for index_q, row_q in self.layers.get_data_layer('pipelines').iterrows():
            network_constraints_list = []
            for index_p, row_p in self.layers.get_data_layer('pipelines').iterrows():
                if row_p['sowing_dates'] >= row_q['sowing_dates'] - delta and row_p['sowing_dates'] <= row_q['sowing_dates'] + delta:
                    network_constraints_list.append(row_p['canal_id'])
            network_constraints_matrix.append(network_constraints_list)
        return network_constraints_matrix 
    
    def estimated_cluster_yield(self):
        yield_series = []
        for p in self.layers.get_column('pipelines', 'sowing_dates'):
            self.csm.simulate_canopy_cover(p)
            self.csm.simulate_ndvi()
            yield_series.append(self.csm.estimate_yield()/100)
        self.layers.add_column('pipelines', yield_series, 'yield')
        return sum(self.layers.get_column('pipelines', 'yield'))
    
    def show(self):
        self.layers.plot(layer_name='plots', column4color='canal_id', alpha=0.8)
        self.layers.show_data_layer(layer_name='plots')
        self.layers.show_data_layer(layer_name='pipelines')
        self.layers.show(layer_name='plots') 
    
    def fitness_sowing_dates_distribution(self):
        self.verify_irrigation_network_constraints()
        return self.estimated_cluster_yield() - sum(self.layers.get_column('pipelines', 'remaining'))
        
    def verify_irrigation_network_constraints(self):
        remaining_list = []
        delta_list = self.get_delta_list()
        for list_voisin in delta_list:
            list_score = 0
            for canal in list_voisin:
                splited_canal = re.findall('[A-Z]*\d+', canal)
                next_branch = ""
                total_remaining = 0
                for p in splited_canal:
                    temp_canals_list = list_voisin
                    temp_canals_list = list(map(lambda x: re.sub('-', '', x), temp_canals_list))
                    remaining = 0
                    next_branch += p
                    if self.layers.get_data_layer('pipelines')[self.layers.get_column('pipelines', 'canal_id_i') == next_branch]['capacity'].shape[0] > 0:
                        #print('verify ', next_branch)
                        if next_branch in temp_canals_list:
                            temp_canals_list.remove(next_branch)
                        common_canal_capacity = self.layers.get_data_layer('pipelines')[self.layers.get_column('pipelines', 'canal_id_i') == next_branch].capacity.to_numpy()[0]
                        activated_canals_sum = 0
                        for q in temp_canals_list:
                            if next_branch in q:
                                activated_canals_sum += self.layers.get_data_layer('pipelines')[self.layers.get_column('pipelines', 'canal_id_i') == q].capacity.to_numpy()[0]
                        
                        remaining = common_canal_capacity - activated_canals_sum
                        total_remaining += remaining
                list_score += total_remaining
                #print("List score:", list_score) 
            
            remaining_list.append(list_score)
            self.layers.add_column('pipelines', remaining_list, 'remaining')
            #print("Total remaining:", total_remaining)


import time
import os
import re
from stable_baselines3 import PPO, DQN  
import gym
from r3 import R3
from stable_baselines3.common.env_checker import check_env

def main():
    ti = time.time()
    
    #left, down, up, right
    #environment = Environment('CartPole-v0')
    #environment.render()
    
    env = R3(fitness_threshold=1000)
    """print(env.observation_space.sample().shape)
    env.sow()
    print(env.layers.get_column('pipelines', 'sowing_dates').to_numpy().shape)"""
    
    #check_env(env)
    models_dir = "models/DQN"
    logdir = "logs"

    if not os.path.exists(logdir):
        os.makedirs(logdir)
    
    if not os.path.exists(models_dir):
        os.makedirs(models_dir)
    
    model = DQN("MlpPolicy", env, verbose=1, tensorboard_log=logdir)
    
    EPISODES = 10
    TIMESTEPS = 100
    for episode in range(EPISODES):
        model.learn(total_timesteps=TIMESTEPS, reset_num_timesteps=False, tb_log_name="DQN")
        model.save(f'{models_dir}/' + f'{TIMESTEPS*episode}')
    """obs = env.reset()
    x = []
    for i in range(10):
        action, _states = model.predict(obs, deterministic=True)
        obs, reward, done, info = env.step(action)
        x.append(reward)
        env.render()
        if done:
            obs = env.reset()

    print(x)"""
    env.close()
    
    
    print(time.time() - ti)


if __name__ == '__main__':
    main()


from math import exp
from .dataframe import DataFrame
import numpy as np
import matplotlib.pyplot as plt
from pandas import Series
from .chart import Chart
import seaborn as sns

class CSM:
    
    def __init__(self):
        self.crop_type = 'Wheat'
        self.season_length = 242
        self.climate_dataframe = self.read_climate_dataframe()
        self.monitoring = DataFrame(Series([i+1 for i in range(self.season_length)]), ['day'], data_type='list')
        
        self.CC0 = 4.5
        self.CCx = 89.33
        self.CGC = 0.0089
        self.CDC = 0.145
        self.CCx_2 = self.CCx / 2
        self.Tupper = 33
        self.Tbase = 5
        self.CGDD_sowing = 82
        
        # hectare
        self.area = 100
        
        
        # crop characteristics #
        # 0.32 m-3/3-3 ou % theta_fc
        self.wc_field_capacity = 0.32
        
        # 0.17 m-3/3-3 ou % theta_fc
        # it is the water quantity below which the crop can no longer extract water, it is the separtion of
        # AW or TAW and NAW
        self.wc_wilting_point = 0.17
        
    def cc_equation1(self, t):
        return self.CC0 * exp(t * self.CGC)
    
    def cc_equation2(self, t):
        return self.CCx - (0.25 * exp(-t * self.CGC) * (self.CCx**2)/(self.CC0))
     
    def cc_equation3(self, t):
        return self.CCx * (1 - (0.05 * (exp((3.33 * self.CDC) * t/(self.CCx + 2.29)) - 1))) 
    
    def simulate_canopy_cover(self, offset=0):
        Tmb = np.zeros((self.season_length,))
        ti = np.zeros((self.season_length,))
        CC = np.zeros((self.season_length,))
        Eq1 = np.zeros((self.season_length,))
        Eq2 = np.zeros((self.season_length,))
        Eq3 = np.zeros((self.season_length,))
        for day in range(offset, self.season_length):
                if (self.climate_dataframe[day] < 5):
                    Tmb[day] = 0 
                else:
                    if (self.climate_dataframe[day] >= self.Tupper):
                        Tmb[day] = self.Tupper - self.Tbase
                    else:
                        Tmb[day] = self.climate_dataframe[day] - self.Tbase
        
        ti[offset] = Tmb[offset]
        for k in range((offset + 1), 242):
            ti[k] = Tmb[k] + ti[k - 1]

        t0_all = np.argwhere(ti >= self.CGDD_sowing)
        t0 = t0_all[0]
        

        for i in range(offset, t0[0]):
            CC[i] = 0

        CC[t0[0]] = self.CC0
        ti[t0[0]] = 0
 
        
        for p in range((t0[0] + 1), 242):
            ti[p] = Tmb[p] + ti[p - 1]

        for m in range((t0[0] + 1), 242):
            Eq1[m] = self.cc_equation1(ti[m])
            
        
        for m in range((t0[0] + 1),242):
            Eq1[m] = self.cc_equation1(ti[m])
            Eq2[m] = self.cc_equation2(ti[m])
            Eq2[m] = Eq2[m].round(2)

        p1 = np.argwhere(Eq1 >= self.CCx_2)
        phase1 = p1[0][0]

        for ii in range((t0[0] + 1), phase1):
            CC[ii] = Eq1[ii]

        p2 = np.argwhere(Eq2 >= self.CCx)
        phase2 = p2[0][0]
        
        for jj in range(phase1, phase2):
            CC[jj] = Eq2[jj]

        ti[phase2] = 0
        CC[phase2] = self.CCx

        for kk in range((phase2 + 1), 242):
            ti[kk] = Tmb[kk] + ti[kk - 1]
            Eq3[kk] = self.cc_equation3(ti[kk])
            if (Eq3[kk] >= 0):
                CC[kk] = Eq3[kk]
            else:
                CC[kk] = 0
        
        for kk in range((phase2 + 1), 242):
            if (Eq3[kk] < self.CCx_2):
                day_final = kk - 1
                break
            
        self.monitoring.add_column(CC, 'cc')   
        return CC
    
    def simulate_fc(self):
        self.monitoring.add_transformed_columns('fc', 'cc/100')

    def simulate_ndvi(self):
        self.monitoring.add_transformed_columns('ndvi', '(cc/118)+0.14')
    
    def simulate_kcb(self):
        self.monitoring.add_transformed_columns('k_cb', '(1.64*ndvi)-0.2296')

    def simulate_ke(self):
        self.monitoring.add_transformed_columns('k_e', '[0.2 (1−fc)]')

    def simulate_et0(self, method='pm'):
        self.monitoring.add_transformed_columns('et_0', '(1.64*ndvi)-0.2296')

    def simulate_etc(self, method='double'):
        self.monitoring.add_transformed_columns('et_c', '[(1.64 * NDVI)-0.2296]+[0.2 * (1 - fc)]*et_0')

    def simulate_p(self, method='pm'):
        self.monitoring.add_transformed_columns('p', '0.55+0.04*(5-et_c)')

    def simulate_raw(self, method='pm'):
        self.monitoring.add_transformed_columns('raw', '0.55+0.04*(5-et_c)')

    def simulate_taw(self, method='pm'):
        self.monitoring.add_transformed_columns('taw', '1000*(0.32-0.17)*zr')

    def estimate_yield(self, method='last_10_ndvi'):
        ndvi_list = self.monitoring.get_column('ndvi')
        if method == 'max_ndvi':
            ndvi_max = float(max(ndvi_list))
            estimated_yield = 23.69*ndvi_max - 13.87
        elif method == 'last_10_ndvi':
            ndvi_list = list(ndvi_list)
            sum_of_last_10_ndvi = sum([float(ndvi_list[153-i]) for i in range(10)])
            estimated_yield = 1.79*sum_of_last_10_ndvi - 8.62
        
        return estimated_yield*self.area
    
    def monitor(self):
        self.monitoring.show()
        fig, axes = plt.subplots(1, 2, sharex=True, figsize=(10,5))
        fig.suptitle('Visual simulation')
        
        # CC
        sns.lineplot(ax=axes[0], x=self.monitoring.get_dataframe().index, y=self.monitoring.get_column('cc').values)
        axes[0].set_title(self.monitoring.get_column('cc').name)
        
        # fc
        #sns.lineplot(ax=axes[1], x=self.monitoring.get_dataframe().index, y=self.monitoring.get_column('fc').values)
        #axes[1].set_title(self.monitoring.get_column('fc').name)
        
        # NDVI
        sns.lineplot(ax=axes[1], x=self.monitoring.get_dataframe().index, y=self.monitoring.get_column('ndvi').values)
        axes[1].set_title(self.monitoring.get_column('ndvi').name)
        plt.show() 
        
    
    def read_climate_dataframe(self):
        data = DataFrame('mean_temperature.csv')
        data.keep_columns(['t_mean'])
        return data.get_column_as_list('t_mean')


import time
import os
import sys


def main():
    ti = time.time()
    data = DataFrame()
    data.load_dataset('iris')
    y =  data.get_column('target')
    data.drop_column('target')
    
    # decision tree model
    model = Model(data_x=data.get_dataframe(), data_y=y, model_type='dt', training_percent=0.8)
    
    # train the model
    model.train()
    
    # get all classification evaluation metrics
    model.report()
    
    #get the cross validation
    model.cross_validation(5)
   
    print(time.time() - ti)


if __name__ == '__main__':
    main()



import gym
import numpy as np

class Environment:
    
    def __init__(self, environment_name="FrozenLake-v0", is_slippery=True):
        """The environment constructor"""
        if environment_name == "FrozenLake-v0":
            self.__environment = gym.make(environment_name, is_slippery=is_slippery)
        else:
            self.__environment = gym.make(environment_name)
        self.__status = self.reset()
        
        
    def describe(self):
        print("Environement states: ", self.__environment.observation_space)
        print("Availble actions: ", self.__environment.action_space)
        # env.P[state][action] to get one transition probability
        # The result is in the form of: [(transition probability, next state, reward, Is terminal state?)]
        print(gym.spaces)
        print(isinstance(self.__environment.observation_space, gym.spaces.box.Box))
        #print("Transition probabilities matrix ", self.__environment.P)
        
    def get_transition_probability_state_action(self, state, action):
        """[(transition probability, next state, reward, Is terminal state?)]"""
        return self.__environment.P[state][action]
        
    def render(self):
        """show the environment"""
        self.__environment.render()
        
    def reset(self):
        """resetting puts our agent back to the initial state"""
        self.__status = self.__environment.reset()
        
    def close(self):
        """resetting puts our agent back to the initial state"""
        self.__environment.close()
        
    def step(self, action=None):
        """ step
        
        take a given action
        The step function return : (next_state, reward, done, info)
        if no action, a random action will be sampled
        """
        if action is None:
            action = self.__environment.action_space.sample()    
        self.__status = self.__environment.step(action)
    
    def status(self):
        """ The status is in the form : (next_state, reward, done, info) """
        return self.__status
    
    def is_terminal_state(self):
        """ check if the agent is in a terminal state"""
        if self.__status[2]:
            return True
        return False 
    
    def random_episode(self, number_of_times_steps=10):
        """random trajectory from an initial state to a final state possible only in an episodic 
        environement and not continous one
        """
        t = 0
        print('Time Step 0 :')
        self.render()
        self.step()
        while not self.is_terminal_state():
            self.render()
            print ('Time Step {} :'.format(t+1))
            self.step()
            t += 1
            
    def random_horizion(self, number_of_times_steps=10):
        """random trajectory starting from an initial state to given number of time steps
        """
        for i in range(number_of_times_steps):
            self.render()
            self.step()
            
    def all_environments(self):
        print(gym.envs.registry.all())
        
    def action_space_cardinal(self):
        return self.__environment.action_space.n
    
    def state_space_cardinal(self):
        return self.__environment.observation_space.n
    
    def value_iteration(self, num_iterations=1000, threshold=1e-20, gamma=1.0):
        value_table = np.zeros(self.state_space_cardinal())
        
        for i in range(num_iterations):
            updated_value_table = np.copy(value_table) 
            for s in range(self.state_space_cardinal()):
                Q_values = [sum([prob*(r + gamma * updated_value_table[s_])
                                for prob, s_, r, _ in self.get_transition_probability_state_action(s,a)])
                                    for a in range(self.action_space_cardinal())]
                                            
                value_table[s] = max(Q_values)
            
            if (np.sum(np.fabs(updated_value_table - value_table)) <= threshold):
                break
        
        return value_table
    
    def extract_policy(self, value_table, gamma = 1.0):
        policy = np.zeros(self.state_space_cardinal()) 
    
        for s in range(self.state_space_cardinal()):
            
            Q_values = [sum([prob*(r + gamma * value_table[s_])
                                for prob, s_, r, _ in self.get_transition_probability_state_action(s, a)]) 
                                    for a in range(self.action_space_cardinal())]
                    
            policy[s] = np.argmax(np.array(Q_values)) 
        
        return policy
    
    
    def run_policy(self, policy):
        self.reset()
        state = self.status()
        print(policy)
        for p in range(50):
            self.step(int(policy[state]))
            state = self.status()[0]
            self.render()




import seaborn as sns
from matplotlib import pyplot as plt
import plotly.graph_objects as go
import plotly.express as px
import matplotlib.ticker as plticker
import numpy as np
import pandas as pd



class Chart:
    chart_type_list = ['line', 'bar', 'box', 'swarm', 'strip_swarm', 'count', 'scatter', 'dist', 'point', 'pair',
                  'correlation_map', 'reg', 'heat_map']

    def __init__(self, dataframe=None, column4x=None, chart_type='pair', group_by=None, columns_names_list=None, plotly=False):
        self.dataframe = dataframe
        if column4x is None:
            self.column4x = dataframe.index
        else:
            self.column4x = column4x
        self.chart_type = chart_type
        self.group_by = group_by
        self.columns_names_list = columns_names_list
        self.plotly = plotly
        sns.set_theme(color_codes=True)
        

    def add_data_to_show(self, data_column=None, column4hover=None, column4size=None, y_column=None, color=None):
        print(self.chart_type)
        if self.plotly == True:
            if self.chart_type == self.chart_type_list[0]:
                self.fig = px.line(self.dataframe, x=self.column4x, y=data_column, color=self.group_by, hover_name=column4hover)
            elif self.chart_type == self.chart_type_list[1]:
                self.ax = sns.barplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
                loc = plticker.MultipleLocator(base=1.0) # this locator puts ticks at regular intervals
                self.ax.xaxis.set_major_locator(loc)
            elif self.chart_type == self.chart_type_list[2]:
                sns.boxplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[3]:
                sns.swarmplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[4]:
                sns.stripplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[5]:
                sns.countplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[6]:
                sns.regplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[7]:
                sns.distplot(self.dataframe.column, kde=False, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[8]:
                sns.pointplot(x="class", y="survived", hue=self.group_by, data=self.dataframe, palette={"male": "g", "female": "m"},
                            markers=["^", "o"], linestyles=["-", "--"], capsize=.2)
            elif self.chart_type == self.chart_type_list[9]:
                dataframe = DataFrame()
                dataframe.set_data_frame(self.dataframe)
                self.fig = px.scatter_matrix(self.dataframe, dimensions=dataframe.get_columns_names(),
                    color=self.group_by)
            elif self.chart_type == self.chart_type_list[10]:
                sns.clustermap(self.dataframe.corr(), cmap=sns.diverging_palette(230, 20, as_cmap=True), annot=True,
                            fmt='1%',
                            center=0.0)
            elif self.chart_type == self.chart_type_list[11]:
                self.fig = px.scatter(self.dataframe, x=self.column4x, y=data_column, color=self.group_by, size=column4size, hover_name=column4hover)
        else:
            #sns.set_style("whitegrid")
            if self.chart_type == self.chart_type_list[0]:
                self.ax = sns.lineplot(data=self.dataframe, x=self.column4x, y=data_column, markers=True, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[1]:
                sns.set_theme(style="whitegrid")
                self.ax = sns.barplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[2]:
                self.ax = sns.boxplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[3]:
                self.ax = sns.swarmplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[4]:
                self.ax = sns.stripplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[5]:
                self.ax = sns.countplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[6]:
                self.ax = sns.regplot(data=self.dataframe, x=self.column4x, y=data_column, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[7]:
                self.ax = sns.distplot(self.dataframe.column, kde=False, hue=self.group_by)
            elif self.chart_type == self.chart_type_list[8]:
                self.ax = sns.pointplot(x="class", y="survived", hue=self.group_by, data=self.dataframe, palette={"male": "g", "female": "m"},
                            markers=["^", "o"], linestyles=["-", "--"], capsize=.2)
            elif self.chart_type == self.chart_type_list[9]:
                self.ax = sns.pairplot(self.dataframe, hue=self.group_by, vars=self.columns_names_list)
            elif self.chart_type == self.chart_type_list[10]:
                self.ax = sns.clustermap(self.dataframe.corr(), annot=True, center=0.0)
            elif self.chart_type == self.chart_type_list[11]:
                self.ax = sns.jointplot(x=data_column, y=y_column, data=self.dataframe, kind="reg", color=color)
                #self.ax = sns.scatterplot(data=self.dataframe, x=data_column, y=y_column)
            elif self.chart_type == self.chart_type_list[12]:
                # Compute the correlation matrix
                corr = self.dataframe.corr()
                # Generate a mask for the upper triangle
                mask = np.triu(np.ones_like(corr, dtype=bool))
                # Generate a custom diverging colormap
                cmap = sns.diverging_palette(230, 20, as_cmap=True)
                # Draw the heatmap with the mask and correct aspect ratio
                self.ax = sns.heatmap(corr, mask=mask, cmap=cmap, vmax=.3, center=0,
                            square=True, linewidths=.5, cbar_kws={"shrink": .5})
            return self.ax
    
    def plot_on_map(self,
                    iso_locations_column=None,
                    circle_size_column=None,
                    animation_frame_column=None,
                    hover_name_column=None, 
                    projection='natural earth',
                    scope='world'):
        self.fig = px.scatter_geo(
            self.dataframe,
            locations=iso_locations_column,
            size=circle_size_column,
            animation_frame=animation_frame_column,
            hover_name=hover_name_column,
            color=self.group_by,
            projection=projection,
            scope=scope,
            )
        
    def plot_colored_map(self,
                    iso_locations_column=None,
                    color_column=None,
                    animation_frame_column="Year",
                    scope='world',
                    hover_name_column=None, # column to add to hover information
                    ):# column on which to animate):
       self.fig = px.choropleth(
                    self.dataframe,
                    locations=iso_locations_column,
                    scope=scope,
                    color=color_column, # lifeExp is a column of gapminder
                    hover_name=hover_name_column, # column to add to hover information
                    color_continuous_scale=px.colors.sequential.Plasma,
                    animation_frame=animation_frame_column,
                    projection='natural earth'# column on which to animate
                    )
                
        
    def show(self):
        if self.plotly:
            self.fig.show()
        else:
            plt.show()

    def config(self,
               title="", 
               x_label="X", 
               y_label="Y", 
               x_limit_i=None, 
               x_limit_f=None, 
               y_limit_i=None, 
               y_limit_f=None, 
               interval=None,
               x_rotation_angle=90,
               y_rotation_angle=0,
               titile_font_size=29,
               x_label_font_size=13,
               y_label_font_size=13,
               x_font_size=11,
               y_font_size=11,
               ):
        if self.plotly:
            self.fig.update_layout(
            # add a title text for the plot
            title_text = title,
            # set projection style for the plot
            #geo = dict(projection={'type':'natural earth'}
            ) # by default, projection type is set to 'equirectangular'
        else:
            plt.title(title)
            plt.xlim(x_limit_i, x_limit_f)
            plt.ylim(y_limit_i, y_limit_f)
            plt.xticks(rotation=x_rotation_angle, fontsize=x_font_size)
            plt.yticks(rotation=y_rotation_angle, fontsize=y_font_size)
            self.ax.set_title(title,fontsize=titile_font_size)
            self.ax.set_xlabel(x_label,fontsize=x_label_font_size)
            self.ax.set_ylabel(y_label,fontsize=y_label_font_size)
            
            if interval is not None:
                loc = plticker.MultipleLocator(base=interval) # this locator puts ticks at regular intervals
                self.ax.xaxis.set_major_locator(loc)


    def save(self, chart_path="output.png", transparent=False):
        if self.plotly is True:
            self.fig.savefig(chart_path, transparent=transparent, bbox_inches = 'tight', dpi=600)
        else:
            self.ax.savefig(chart_path, transparent=transparent, bbox_inches = 'tight', dpi=600)
